# pylint: disable=no-member

import os
from typing import Any, Union
import cv2 as cv
import pytesseract
from robot.api.deco import keyword
from ..Ibasic.IBasic import IBasic


class Tesseract(IBasic):
    """Class representing Tesseract and its methods.
    This class is not instantiated itself, but inherited into main WatchUI module class.
    Args:
        Basics (object): interface class
    """

    @keyword
    def image_to_string(
        self,
        path: str,
        oem: str = "3",
        psm: str = "3",
        language: str = "eng",
        path_to_tesseract="",
    ) -> Union[bytes, str]:
        """Converts provided image to string and returns.
        Args:
            path (str): path to the image
            oem (str): Engine mode. Tesseract has several engine modes with different performance and speed.
            psm (str): Page Segmentation Mode (--psm). That affects how Tesseract splits image in lines of text and words. Pick the one which works best for you.
            language (str): [description]
            path_to_tesseract (str): path to installed tesseract binary
        Returns:
            str: OCRed text from image
        """
        self.check_image_exists(path)

        old_img: Any = cv.imread(path)
        pytesseract.pytesseract.tesseract_cmd = self.check_tess_path(path_to_tesseract)
        custom_oem_psm_config: str = rf"--oem {oem} --psm {psm}"

        text: Union[bytes, str] = pytesseract.image_to_string(
            old_img, config=custom_oem_psm_config, lang=language
        )
        return text

    @keyword
    def image_area_on_text(
        self,
        path: str,
        *coordinates,
        oem: str = "3",
        psm: str = "3",
        language: str = "eng",
        path_to_tesseract: str = "",
    ) -> Union[list[str], str]:
        """Convert to text only the selected area of the image and return.
        Args:
            path (str): path to the image
            oem (str): Engine mode. Tesseract has several engine modes with different performance and speed.
            psm (str): Page Segmentation Mode (--psm). That affects how Tesseract splits image in lines of text and words. Pick the one which works best for you.
            language (str): [description]
            path_to_tesseract (str): path to the installed tesseract library
        Returns:
            Union[list[str], str]: returned text
        """
        string_list = []
        text: Union[bytes, str]
        old_img: Any = cv.imread(path)
        len_coordinates: int = len(coordinates)

        self.check_if_args_has_ok_numbers(*coordinates, need_numbers=4)
        if len_coordinates / 4 == 1:
            crop_img: Any = old_img[
                int(coordinates[1]) : int(coordinates[3]),
                int(coordinates[0]) : int(coordinates[2]),
            ]
            pytesseract.pytesseract.tesseract_cmd = self.check_tess_path(
                path_to_tesseract
            )
            custom_oem_psm_config: str = rf"--oem {oem} --psm {psm}"
            text = pytesseract.image_to_string(
                crop_img, config=custom_oem_psm_config, lang=language
            )
            if not isinstance(text, str):
                raise TypeError(
                    "Tesseract OCRed image and returned data as bytes. We need <str> type."
                )
            stripped_text: str = os.linesep.join(
                [s for s in text.splitlines() if s]
            )  # delete blank line space
            return stripped_text

        num_coordinates: float = len_coordinates / 4
        a: int = 0
        i: int = 0
        while i < num_coordinates:
            x1 = coordinates[0 + a]
            y1 = coordinates[1 + a]
            x2 = coordinates[2 + a]
            y2 = coordinates[3 + a]
            crop_img = old_img[int(y1) : int(y2), int(x1) : int(x2)]
            pytesseract.pytesseract.tesseract_cmd = self.check_tess_path(
                path_to_tesseract
            )
            custom_oem_psm_config = rf"--oem {oem} --psm {psm}"
            text = pytesseract.image_to_string(
                crop_img, config=custom_oem_psm_config, lang=language
            )
            if not isinstance(text, str):
                raise TypeError(
                    "Tesseract OCRed image and returned data as bytes. We need <str> type."
                )

            string_list.append(text)
            i += 1
            a += 4
        return string_list



# pylint: disable=invalid-name, expression-not-assigned, unbalanced-tuple-unpacking, too-many-arguments, too-many-locals, no-member
import time
from typing import Any, Optional, Union

from robot.api.deco import keyword
import cv2 as cv
import imutils
from numpy import ndarray
from skimage.metrics import structural_similarity
from ..Ibasic.IBasic import IBasic


class Image(IBasic):
    """Class representing the Image and methods for RF keywords regarding
    comparing the images.
    This class is not instantiated itself.
    Args:
        Basics (object): interface class
    """

    save_folder = ""
    ssim = 1.0
    image_format = "png"

    def _do_checks(self, expected_returned_items: int, **kwargs) -> tuple[Any, ...]:
        keys: list[Any] = list(kwargs.keys())

        checked_save_folder: Optional[str] = (
            self.check_dir(kwargs["save_folder"]) if "save_folder" in keys else None
        )
        ssim: Optional[float] = (
            self.check_ssim(kwargs["ssim"]) if "ssim" in keys else None
        )
        img_format: Optional[str] = (
            self.check_image_format(kwargs["image_format"])
            if "image_format" in keys
            else None
        )
        self.check_image_exists(
            kwargs["base_image_path"]
        ) if "base_image_path" in keys else None
        self.check_image_exists(
            kwargs["compared_image_path"]
        ) if "compared_image_path" in keys else None

        output: list[Union[str, float]] = []
        checked_save_folder is not None and output.append(checked_save_folder)  # type: ignore
        ssim is not None and output.append(ssim)  # type: ignore
        img_format is not None and output.append(img_format)  # type: ignore

        if len(output) != expected_returned_items:
            raise IndexError(
                "Expected items to be returned by the method differs from actual."
            )

        return tuple(output)

    @staticmethod
    def _get_images(base_image_path: str, compared_image_path: str) -> tuple[Any, ...]:
        img1: Any = cv.imread(base_image_path, 1)
        img2: Any = cv.imread(compared_image_path, 1)

        return tuple([img1, img2])

    @staticmethod
    def _convert_to_grey(base_image: Any, target_image: Any) -> tuple[Any, ...]:
        gray_img1: Any = cv.cvtColor(base_image, cv.COLOR_BGR2GRAY)
        gray_img2: Any = cv.cvtColor(target_image, cv.COLOR_BGR2GRAY)

        return tuple([gray_img1, gray_img2])

    @staticmethod
    def _compute_score(
        gray_base_img: Any, gray_targed_img: Any
    ) -> tuple[float, Union[ndarray, Any]]:
        score: float
        diff: Union[ndarray, Any]
        (score, diff) = structural_similarity(gray_base_img, gray_targed_img, full=True)
        diff = (diff * 255).astype("uint8")

        if not isinstance(score, float):
            raise TypeError(
                f"Returned score must be of <float> type. Received {type(score)}"
            )

        return score, diff

    def _write_and_log(
        self,
        score: float,
        ssim: float,
        checked_save_folder: str,
        img_format: str,
        base_image: Any,
        target_image: Any,
    ) -> Any:
        time_: str = str(time.time())
        url: str = f"{checked_save_folder}/Img{time_}{img_format}"
        # Show image
        if score < ssim:
            cv.imwrite(url, target_image)
            self.set_log_message(
                work_object="Image", type_of_messages="Error", path_to_image=url
            )
        else:
            img_diff: Any = cv.hconcat([base_image, target_image])
            cv.imwrite(url, img_diff)
            self.set_log_message(
                work_object="Image", type_of_messages="Info", path_to_image=url
            )

    @staticmethod
    def _get_contours(diff: Union[ndarray, Any]) -> Any:
        thresh: Any = cv.threshold(diff, 0, 255, cv.THRESH_BINARY_INV | cv.THRESH_OTSU)[
            1
        ]
        cnts: Any = cv.findContours(
            thresh.copy(), cv.RETR_EXTERNAL, cv.CHAIN_APPROX_SIMPLE
        )
        return imutils.grab_contours(cnts)

    @keyword
    def compare_images(
        self,
        base_image_path: str,
        compared_image_path: str,
        save_folder=save_folder,
        ssim=ssim,
        image_format=image_format,
    ) -> None:
        """Compares image on `compared_image_path` against base image on `base_image_path`.
        Args:
            base_image_path (str): path to base image
            compared_image_path (str): path to compared image
            save_folder (str): path to the save folder
            ssim (float): SSIM threshold value
            image_format (str): image format, e.g. "png", "jpg"
        Raises:
            TypeError:
        """
        checked_save_folder, ssim, img_format = self._do_checks(
            3,
            save_folder=save_folder,
            base_image_path=base_image_path,
            compared_image_path=compared_image_path,
            ssim=ssim,
            image_format=image_format,
        )

        # Compare image
        img1, img2 = self._get_images(base_image_path, compared_image_path)

        # convert to grey
        gray_img1, gray_img2 = self._convert_to_grey(img1, img2)

        # SSIM diff Img
        score, diff = self._compute_score(gray_img1, gray_img2)

        # Threshold diff Img
        cnts: Any = self._get_contours(diff)

        # Create frame in diff area
        for c in cnts:
            (x, y, w, h) = cv.boundingRect(c)
            cv.rectangle(img1, (x, y), (x + w, y + h), (0, 0, 255), 2)
            cv.rectangle(img2, (x, y), (x + w, y + h), (0, 0, 255), 2)

        self._write_and_log(score, ssim, checked_save_folder, img_format, img1, img2)

    @keyword
    def compare_screen_without_areas(
        self,
        base_image_path: str,
        compared_image_path: str,
        *args,
        save_folder=save_folder,
        ssim=ssim,
        image_format=image_format,
    ) -> None:
        """Compares two images without selected areas of the image.
        Args:
            base_image_path (str): path to the base image
            compared_image_path (str): path to the compared image
            save_folder (str): path to the save folder
            ssim (float): SSIM threshold value
            image_format (str): format of the image, e.g. "png", "jpg"
        """
        checked_save_folder, my_ssim, img_format = self._do_checks(
            3,
            save_folder=save_folder,
            base_image_path=base_image_path,
            compared_image_path=compared_image_path,
            ssim=ssim,
            image_format=image_format,
        )

        lt: int = len(args)
        img1, img2 = self._get_images(base_image_path, compared_image_path)

        self.check_if_args_has_ok_numbers(*args, need_numbers=4)

        x: float = lt / 4
        i: int = 0
        a: int = 0
        while i < x:
            color: tuple[int, int, int] = (0, 0, 0)
            x1: int = int(args[0 + a])
            y1: int = int(args[1 + a])
            x2: int = int(args[2 + a])
            y2: int = int(args[3 + a])

            cv.rectangle(img1, (x1, y1), (x2, y2), color, -1)
            cv.rectangle(img2, (x1, y1), (x2, y2), color, -1)
            a += 4
            i += 1

        # convert to grey
        gray_img1, gray_img2 = self._convert_to_grey(img1, img2)

        # SSIM diff Img
        score, diff = self._compute_score(gray_img1, gray_img2)

        # Threshold diff Img
        cnts: Any = self._get_contours(diff)

        # Create frame in diff area
        for c in cnts:
            (x, y, w, h) = cv.boundingRect(c)
            cv.rectangle(img1, (x, y), (x + w, y + h), (0, 0, 255), 2)
            cv.rectangle(img2, (x, y), (x + w, y + h), (0, 0, 255), 2)

        # Show image
        self._write_and_log(score, my_ssim, checked_save_folder, img_format, img1, img2)

    @keyword
    def crop_image(
        self,
        path: str,
        x1: int,
        y1: int,
        x2: int,
        y2: int,
        save_folder=save_folder,
        image_format=image_format,
    ) -> None:
        """Creates cropped image and saves it.
        Args:
            path (str): path to the image
            x1 (int): x1 coords
            y1 (int): y1 coords
            x2 (int): x2 coords
            y2 (int): y2 coords
            save_folder (str): path to save folder
            image_format (str): format of the saved image, e.g. "png", "jpg"
        """
        checked_save_folder, img_save_format = self._do_checks(
            2, save_folder=save_folder, image_format=image_format, base_image_path=path
        )

        img: Any = cv.imread(path, 1)
        crop_img: Any = img[
            int(x1) : int(x2), int(y1) : int(y2)
        ]  # Crop from {x, y, w, h } => {0, 0, 300, 400}

        # Save image
        url: str = f"{checked_save_folder}/Img{str(time.time())}{img_save_format}"
        cv.imwrite(url, crop_img)
        self.set_log_message(
            work_object="Image", type_of_messages="Info", path_to_image=url
        )

    @keyword
    def rotate_image(
        self,
        path: str,
        screen_name="rotate_img",
        angle=0,
        save_folder=save_folder,
        image_format=image_format,
    ) -> Any:
        """Creates rotated version of the source image and saves it.
        Args:
            path (str): path to the source image
            screen_name (str): name of the screen
            save_folder (str): path to the save folder
            rotate (int): [description]
            image_format (str): [description]
        Raises:
            AssertionError: [description]
        """
        if angle not in [0, 1, 2]:
            raise ValueError(
                "Value of 'rotate' argument can only be of type <Literal[0, 1, 2]>"
            )
        checked_save_folder, img_format = self._do_checks(
            2, save_folder=save_folder, image_format=image_format, base_image_path=path
        )

        img: Any = cv.imread(path)
        save_path: str = f"{checked_save_folder}/{screen_name}{img_format}"
        rotate_image: Any

        if angle == 0:
            rotate_image = cv.rotate(img, cv.ROTATE_90_CLOCKWISE)
            return cv.imwrite(save_path, rotate_image)
        if angle == 1:
            rotate_image = cv.rotate(img, cv.ROTATE_90_COUNTERCLOCKWISE)
            return cv.imwrite(save_path, rotate_image)
        if angle == 2:
            rotate_image = cv.rotate(img, cv.ROTATE_180)
            return cv.imwrite(save_path, rotate_image)

        return None



# pylint: disable=invalid-name
"""
Basics class
"""
import os
from typing import Literal
from robot.api import logger
from robot.api.deco import not_keyword


class IBasic:
    """
    Interface to be inherited by Classes implementing keyword methods.
    Raises:
        FileExistsError:
        TypeError:
        ValueError:
        AssertionError:
        ValueError:
    """

    def __init__(self, save_folder_path, ssim, format_image, tesseract_path) -> None:
        self.save_folder_path = save_folder_path
        self.ssim_basic = ssim
        self.starts_format_image = format_image
        self.path_to_tesseract_folder = tesseract_path

    @not_keyword
    def check_dir(self, save_folder: str) -> str:
        """Checks given value for `save_folder`, verifies against default and if it
        does not exists, creates it.
        Args:
            save_folder (str): path to save folder
        Returns:
            str: self.save_folder_path
        """
        if save_folder != "":
            self.save_folder_path = save_folder

        if not os.path.exists(self.save_folder_path):
            os.mkdir(self.save_folder_path)

        return self.save_folder_path

    @not_keyword
    @staticmethod
    def check_image_exists(path: str) -> None:
        """Checks, that image file on given `path` exists.
        Args:
            path (str): path to given image file.
        Raises:
            FileExistsError:
        """
        if not os.path.exists(path):
            raise FileExistsError(f"The path: {path}, to the image does not exist.")

    @not_keyword
    def check_ssim(self, ssim: float) -> float:
        """Checks, if `ssim` is of `<float>` type and if it is within <0.0 , 1.0> range.
        Args:
            ssim (float): SSIM value
        Raises:
            TypeError:
            ValueError:
        Returns:
            float: returns the SSIM value
        """
        if not isinstance(ssim, float):
            raise TypeError(
                f"'ssim' argument must of type <float>. Provided value is of type {type(ssim)}"
            )
        if (ssim < 0.0) or (ssim > 1.0):
            raise ValueError(
                f"'ssim' value must be in <0.0 , 1.0> inclusive range. You have provided {ssim}"
            )

        if ssim != 1.0:
            self.ssim_basic = ssim
        return self.ssim_basic

    @not_keyword
    def check_image_format(self, image_format: str) -> str:
        """Checks image format. If "png", returns default, else returns `image_format`.
        Args:
            image_format (str): format of the image, e.g. "png", "jpg", etc.
        Returns:
            str: image format file extension, e.g ".png", ".jpg"
        """
        if str(image_format) == "png":
            return f".{self.starts_format_image}"
        return f".{image_format}"

    @not_keyword
    def check_tess_path(self, path_to_tess: str) -> str:
        """Checks, whether `path_to tess` is equal to expected default. If so,
        returns default value.
        Else returns provided value.
        Args:
            path_to_tess (str): path to tesseract folder
        Returns:
            str: self.path_to_tesseract_folder
        """

        if path_to_tess != "":
            self.path_to_tesseract_folder = path_to_tess
        return self.path_to_tesseract_folder

    @not_keyword
    @staticmethod
    def set_log_message(
        work_object: Literal["Image", "PDF", "Tesseract"] = "Image",
        type_of_messages: Literal["Info", "Error"] = "Info",
        path_to_image: str = "",
    ) -> None:
        """Sets logging message.
        Args:
            work_object (Literal[, optional): What object are we working on. Defaults to "Image".
            type_of_messages (Literal[, optional): Type of logging message. Defaults to "Info".
            path_to_image (str, optional): Path to image to be appended to log message.
            Defaults to "".
        Raises:
            AssertionError:
        """
        if work_object == "Image":
            if type_of_messages == "Info":
                logger.info(
                    f"<p>Image was saved to your output folder </p><img src={path_to_image}></img>",
                    html=True,
                )
            elif type_of_messages == "Error":
                logger.error(
                    f"<p>Images arent same</p><img src={path_to_image}></img>",
                    html=True,
                )
                raise AssertionError("Images are not the same")
        elif work_object == "PDF":
            pass
        elif work_object == "Tesseract":
            pass
        else:
            logger.warn(
                "!!! Bad work object !!! Please contact library owner...", html=True
            )

    @not_keyword
    @staticmethod
    def check_if_args_has_ok_numbers(*args, need_numbers: int = 4) -> bool:
        """Verifies, whether correct number of arguments was provided.
        *Args:
            Arguments to be verified
        Args:
            need_numbers (int, optional): required number of arguments. Defaults to 4.
        Raises:
            ValueError:
        Returns:
            bool: True, if number of `*args` matches the `need_number` value.
            Else throws `ValueError`
        """
        n_arg = len(args)
        logger.info(args)
        if n_arg % need_numbers == 0:
            return True

        logger.error(
            f"You tried to set bad numbers of arguments. You need {need_numbers} arguments",
            html=False,
        )
        raise ValueError(
            f"You tried to set bad numbers of arguments. You need {need_numbers} arguments"
        )



# pylint: disable=invalid-name, line-too-long, too-many-arguments
"""
Library module
"""

from .keywords import Image, Pdf, Tesseract


class WatchUI(Image, Pdf, Tesseract):
    """WatchUI - Custom library for comparing images with use in Robot Framework.
    = Table of Contents =
    - `Usage`
    - `Importing`
    - `Examples`
    - `Keywords`
    = Usage =
    This library allows for automated visual testing of web frontends.
    *IMPORTANT*: When using keywords of this library, please remember,
    that screenshots have to have same resolution!
    = Examples =
    Import library
    | `Library` | <path_to_library file> | outputs_folder= | ssim_basic= | format_image= | tesseract_path=
    Compare Images
    | Compare Images | path1 | path2 | save_folder= | ssim= | format_image= |
    """

    def __init__(
        self,
        outputs_folder="./outputs",
        ssim_basic=1.0,
        format_image="png",
        tesseract_path=r"C:\Program Files\Tesseract-OCR\tesseract.exe",
    ):
        """Library can be imported either with default output folder and set
        lowest limit of difference between images (ssim), or
        you can provide your own values.
        Keyword Arguments:
            outputs_folder {str} -- path, where you want to save images with highlighted
            differences (default: "../Outputs")
            ssim_basic {float} -- threshold value in the interval (0, 1>. Tests are passed,
            if ssim value returned by keyword test functions is bigger than this (default: 1.0)
            format_image {str} -- Format for saving picture/screenshot (png, jpg etc.)
            tesseract_path {str} -- path for tesseract
            Example: format_image=jpg (default: png)
        Examples:
        | =Setting= | =Value= | =Value= | =Value= | =Comment= |
        | Library   | WatchUI |      |  | # Uses default values of keyword arguments |
        | Library   | WatchUI | outputs_folder=<path_to_folder> | | # changes folder to different one |
        | Library   | WatchUI | outputs_folder=<path_to_folder> | ssim_basic=<float> | # changes output folder and ssim threshold |
        """
        super().__init__(outputs_folder, ssim_basic, format_image, tesseract_path)



# pylint: disable=no-member, invalid-name, too-many-arguments
"""
PDF class module
"""
from typing import Any
import fitz
from robot.api.deco import keyword
from ..Ibasic.IBasic import IBasic


class Pdf(IBasic):
    """Class representing PDF and its methods.
    Class is not being instantiated itself, but inherited by main WatchUI.py module.
    Args:
        Basics (object): interface class
    """

    save_folder = ""

    @keyword
    def pdf_to_image(
        self,
        path: str,
        name: str = "img",
        page_number: int = -1,
        zoom: int = 3,
        save_folder=save_folder,
    ) -> None:
        """Converts pdf to image - .png format.
        Args:
            path (str): path to the .pdf file
            save_folder (str): path to the save folder
            name (str): name of the file of the image to be saved
            number_page (str): page from the pdf document to be saved as image.
            if value is `-1`, then saves all pages as separated .png files.
        """
        checked_save_folder: str = self.check_dir(save_folder)
        self.check_image_exists(path)

        doc: Any = fitz.open(path)
        page: Any
        pix: Any
        mat = fitz.Matrix(zoom, zoom)
        if page_number == "-1":
            page_count: int = doc.pageCount
            for x in range(0, page_count):
                page = doc.load_page(x)  # load all pages one by one
                pix = page.get_pixmap(matrix=mat)
                output: str = f"{checked_save_folder}/{name}_{str(x)}.png"
                pix.save(output)
                self.set_log_message(work_object="Image", path_to_image=output)
        else:
            page = doc.load_page(int(page_number))  # number of page
            pix = page.get_pixmap(matrix=mat)
            output = f"{checked_save_folder}/{name}.png"
            pix.save(output)
            self.set_log_message(work_object="Image", path_to_image=output)

    @keyword
    def return_text_from_area(
        self, path: str, page_number: int, x1: int, y1: int, x2: int, y2: int
    ) -> str:
        """Opens provided PDF file and returns text from the given area.
        Args:
            path (str): path to the pdf file
            page_number (int): page number in the pdf file
            x1 (int):
            y1 (int):
            x2 (int):
            y2 (int):
        Returns:
            str: selected text from the area
        """
        self.check_image_exists(path)
        doc: Any = fitz.open(path)
        page: Any = doc[page_number]
        words_list: list[Any] = page.get_text_words()
        text: str = ""
        xy_numbers: int = 1

        for xy in words_list:
            if (
                float(xy[0]) > float(x1)
                and float(xy[1]) > float(y1)
                and float(xy[2]) < float(x2)
                and float(xy[3]) < float(y2)
            ):
                if xy_numbers == len(words_list):
                    text += xy[4]
                else:
                    text += xy[4] + " "
            xy_numbers += 1
        return text

    @keyword
    def should_exist_this_text(self, path: str, page_number: int, text: str) -> bool:
        """Verifies, whether given text is present in the document and given page.
        Args:
            path (str): path to pdf document
            page_number (int): page number
            text (str): text to check for presence
        Returns:
            bool: `True` if present, else `False`
        """
        self.check_image_exists(path)
        doc: Any = fitz.open(path)
        page: Any = doc[page_number]
        text_instances: list[Any] = page.search_for(text)
        if len(text_instances) > 0:
            return True

        return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['WatchUI', 'WatchUI.Ibasic', 'WatchUI.keywords']

package_data = \
{'': ['*']}

install_requires = \
['Pillow',
 'PyMuPDF',
 'imutils',
 'numpy',
 'opencv-python',
 'pandas',
 'pytesseract',
 'robotframework',
 'scikit-image',
 'scikit-learn']

entry_points = \
{'console_scripts': ['docs = tasks:docs', 'test = tasks:test']}

setup_kwargs = {
    'name': 'watchui',
    'version': '2.0.11',
    'description': 'RobotFramework library package for automated visual testing.',
    'long_description': '# WatchUI\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n## [Documentation](https://tesena-smart-testing.github.io/WatchUI/) | [Tesena](https://www.tesena.com/) | [Pypi](https://pypi.org/project/WatchUI/)\n\n## Important notice for users\n\nWatchUI 2.0 brings breaking changes. Dev team decided to streamline the library and focus it solely on the image and text comparison. This allows us to remove the implicit dependency on browser automation libraries - namely SeleniumLibrary, which was implicit part of the library via RF `BuiltIn()` import of the SeleniumLibrary instance.\n\nThis is no longer the case - user of the WatchUI therefore **can and have to choose, what UI automation library will use** and provide screenshots to the WatchUI keywords to be compared. It could be now used with SeleniumLibrary, Browser library, Sikuli, Appium or any other UI library where visual validation is required.\n\nVersion 1.x.x is no longer supported, but it is still available on [Pypi](pip install WatchUI==1.0.11).\n\n### Basic Info\n\nCustom library for works with image, pdf and tesseract with RF.\n\n### Folder structure\n\n```\nWatchUI\n└── .github/workflows           # Folder with CI for github actions\n└── assets                      # Folder with images used for documantation as well as test data\n└── test                        # Folder with example how to write rf test.\n│    └── unit_tests             # Pytest unit test cases\n│    └── test.robot             # File with simeple Robot Framework TCs\n└── WatchUI                     # Folder with WatchUI library\n│    └── WatchUI.py             # Main library file\n│    └── IBasics                # Basic utilities and error handling\n│    └── Keywords               # Keywords for working with images, PDFs and text(tesseract)\n└── README.MD                   # Here you are :-)\n└── setup.py                    # File for easy setup use with pip install .\n```\n\n### Install\n\nYou can find detail in [Documentation](https://procesor2017.github.io/WatchUI/) but basically use pip:\n\n```\npip install WatchUI\n```\n\nor some python dependencies management tools, like [pipenv](https://pipenv.pypa.io/en/latest/) or [poetry](https://python-poetry.org/) and their respective methods of libraries installation.\n\n### Sample results\n\n_Image where the differences are stored + You can see two black box in left corner. These black box are ignored during comparison._\n\n<img src="assets/example-ignore-areas.png">\n\n_The red rectangles outlining missing elements on compared screens_\n\n<img src="assets/example-difference.jpg">\n',
    'author': 'Jan Egermaier',
    'author_email': 'jan.egermaier@tesena.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Tesena-smart-testing/WatchUI',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
